#
# Copyright (C) 2019 EfficiOS Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; only version 2
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

import unittest
import bt2
from bt2 import port as bt2_port


class PortTestCase(unittest.TestCase):
    @staticmethod
    def _create_comp(comp_cls, name=None):
        graph = bt2.Graph()

        if name is None:
            name = 'comp'

        return graph.add_component(comp_cls, name)

    def test_src_add_output_port(self):
        class MySource(
            bt2._UserSourceComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                port = comp_self._add_output_port('out')
                self.assertEqual(port.name, 'out')

        comp = self._create_comp(MySource)
        self.assertEqual(len(comp.output_ports), 1)
        self.assertIs(type(comp.output_ports['out']), bt2_port._OutputPortConst)

    def test_flt_add_output_port(self):
        class MyFilter(
            bt2._UserFilterComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                port = comp_self._add_output_port('out')
                self.assertEqual(port.name, 'out')

        comp = self._create_comp(MyFilter)
        self.assertEqual(len(comp.output_ports), 1)

    def test_flt_add_input_port(self):
        class MyFilter(
            bt2._UserFilterComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                port = comp_self._add_input_port('in')
                self.assertEqual(port.name, 'in')

        comp = self._create_comp(MyFilter)
        self.assertEqual(len(comp.input_ports), 1)
        self.assertIs(type(comp.input_ports['in']), bt2_port._InputPortConst)

    def test_sink_add_input_port(self):
        class MySink(bt2._UserSinkComponent):
            def __init__(comp_self, config, params, obj):
                port = comp_self._add_input_port('in')
                self.assertEqual(port.name, 'in')

            def _user_consume(self):
                pass

        comp = self._create_comp(MySink)
        self.assertEqual(len(comp.input_ports), 1)

    def test_user_src_output_ports_getitem(self):
        class MySource(
            bt2._UserSourceComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                port1 = comp_self._add_output_port('clear')
                port2 = comp_self._add_output_port('print')
                port3 = comp_self._add_output_port('insert')
                self.assertEqual(port3.addr, comp_self._output_ports['insert'].addr)
                self.assertEqual(port2.addr, comp_self._output_ports['print'].addr)
                self.assertEqual(port1.addr, comp_self._output_ports['clear'].addr)

        self._create_comp(MySource)

    def test_user_flt_output_ports_getitem(self):
        class MyFilter(
            bt2._UserFilterComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                port1 = comp_self._add_output_port('clear')
                port2 = comp_self._add_output_port('print')
                port3 = comp_self._add_output_port('insert')
                self.assertEqual(port3.addr, comp_self._output_ports['insert'].addr)
                self.assertEqual(port2.addr, comp_self._output_ports['print'].addr)
                self.assertEqual(port1.addr, comp_self._output_ports['clear'].addr)

        self._create_comp(MyFilter)

    def test_user_flt_input_ports_getitem(self):
        class MyFilter(
            bt2._UserFilterComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                port1 = comp_self._add_input_port('clear')
                port2 = comp_self._add_input_port('print')
                port3 = comp_self._add_input_port('insert')
                self.assertEqual(port3.addr, comp_self._input_ports['insert'].addr)
                self.assertEqual(port2.addr, comp_self._input_ports['print'].addr)
                self.assertEqual(port1.addr, comp_self._input_ports['clear'].addr)

        self._create_comp(MyFilter)

    def test_user_sink_input_ports_getitem(self):
        class MySink(bt2._UserSinkComponent):
            def __init__(comp_self, config, params, obj):
                port1 = comp_self._add_input_port('clear')
                port2 = comp_self._add_input_port('print')
                port3 = comp_self._add_input_port('insert')
                self.assertEqual(port3.addr, comp_self._input_ports['insert'].addr)
                self.assertEqual(port2.addr, comp_self._input_ports['print'].addr)
                self.assertEqual(port1.addr, comp_self._input_ports['clear'].addr)

            def _user_consume(self):
                pass

        self._create_comp(MySink)

    def test_user_src_output_ports_getitem_invalid_key(self):
        class MySource(
            bt2._UserSourceComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                comp_self._add_output_port('clear')
                comp_self._add_output_port('print')
                comp_self._add_output_port('insert')

                with self.assertRaises(KeyError):
                    comp_self._output_ports['hello']

        self._create_comp(MySource)

    def test_user_flt_output_ports_getitem_invalid_key(self):
        class MyFilter(
            bt2._UserFilterComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                comp_self._add_output_port('clear')
                comp_self._add_output_port('print')
                comp_self._add_output_port('insert')

                with self.assertRaises(KeyError):
                    comp_self._output_ports['hello']

        self._create_comp(MyFilter)

    def test_user_flt_input_ports_getitem_invalid_key(self):
        class MyFilter(
            bt2._UserFilterComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                comp_self._add_input_port('clear')
                comp_self._add_input_port('print')
                comp_self._add_input_port('insert')

                with self.assertRaises(KeyError):
                    comp_self._input_ports['hello']

        self._create_comp(MyFilter)

    def test_user_sink_input_ports_getitem_invalid_key(self):
        class MySink(bt2._UserSinkComponent):
            def __init__(comp_self, config, params, obj):
                comp_self._add_input_port('clear')
                comp_self._add_input_port('print')
                comp_self._add_input_port('insert')

                with self.assertRaises(KeyError):
                    comp_self._input_ports['hello']

            def _user_consume(self):
                pass

        self._create_comp(MySink)

    def test_user_src_output_ports_len(self):
        class MySource(
            bt2._UserSourceComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                comp_self._add_output_port('clear')
                comp_self._add_output_port('print')
                comp_self._add_output_port('insert')
                self.assertEqual(len(comp_self._output_ports), 3)

        self._create_comp(MySource)

    def test_user_flt_output_ports_len(self):
        class MyFilter(
            bt2._UserFilterComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                comp_self._add_output_port('clear')
                comp_self._add_output_port('print')
                comp_self._add_output_port('insert')
                self.assertEqual(len(comp_self._output_ports), 3)

        self._create_comp(MyFilter)

    def test_user_flt_input_ports_len(self):
        class MyFilter(
            bt2._UserFilterComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                comp_self._add_input_port('clear')
                comp_self._add_input_port('print')
                comp_self._add_input_port('insert')
                self.assertEqual(len(comp_self._input_ports), 3)

        self._create_comp(MyFilter)

    def test_user_sink_input_ports_len(self):
        class MySink(bt2._UserSinkComponent):
            def __init__(comp_self, config, params, obj):
                comp_self._add_input_port('clear')
                comp_self._add_input_port('print')
                comp_self._add_input_port('insert')
                self.assertEqual(len(comp_self._input_ports), 3)

            def _user_consume(self):
                pass

        self._create_comp(MySink)

    def test_user_src_output_ports_iter(self):
        class MySource(
            bt2._UserSourceComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                port1 = comp_self._add_output_port('clear')
                port2 = comp_self._add_output_port('print')
                port3 = comp_self._add_output_port('insert')
                ports = []

                for port_name, port in comp_self._output_ports.items():
                    ports.append((port_name, port))

                self.assertEqual(ports[0][0], 'clear')
                self.assertEqual(ports[0][1].addr, port1.addr)
                self.assertEqual(ports[1][0], 'print')
                self.assertEqual(ports[1][1].addr, port2.addr)
                self.assertEqual(ports[2][0], 'insert')
                self.assertEqual(ports[2][1].addr, port3.addr)

        self._create_comp(MySource)

    def test_user_flt_output_ports_iter(self):
        class MyFilter(
            bt2._UserFilterComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                port1 = comp_self._add_output_port('clear')
                port2 = comp_self._add_output_port('print')
                port3 = comp_self._add_output_port('insert')
                ports = []

                for port_name, port in comp_self._output_ports.items():
                    ports.append((port_name, port))

                self.assertEqual(ports[0][0], 'clear')
                self.assertEqual(ports[0][1].addr, port1.addr)
                self.assertEqual(ports[1][0], 'print')
                self.assertEqual(ports[1][1].addr, port2.addr)
                self.assertEqual(ports[2][0], 'insert')
                self.assertEqual(ports[2][1].addr, port3.addr)

        self._create_comp(MyFilter)

    def test_user_flt_input_ports_iter(self):
        class MyFilter(
            bt2._UserFilterComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                port1 = comp_self._add_input_port('clear')
                port2 = comp_self._add_input_port('print')
                port3 = comp_self._add_input_port('insert')
                ports = []

                for port_name, port in comp_self._input_ports.items():
                    ports.append((port_name, port))

                self.assertEqual(ports[0][0], 'clear')
                self.assertEqual(ports[0][1].addr, port1.addr)
                self.assertEqual(ports[1][0], 'print')
                self.assertEqual(ports[1][1].addr, port2.addr)
                self.assertEqual(ports[2][0], 'insert')
                self.assertEqual(ports[2][1].addr, port3.addr)

        self._create_comp(MyFilter)

    def test_user_sink_input_ports_iter(self):
        class MySink(bt2._UserSinkComponent):
            def __init__(comp_self, config, params, obj):
                port1 = comp_self._add_input_port('clear')
                port2 = comp_self._add_input_port('print')
                port3 = comp_self._add_input_port('insert')
                ports = []

                for port_name, port in comp_self._input_ports.items():
                    ports.append((port_name, port))

                self.assertEqual(ports[0][0], 'clear')
                self.assertEqual(ports[0][1].addr, port1.addr)
                self.assertEqual(ports[1][0], 'print')
                self.assertEqual(ports[1][1].addr, port2.addr)
                self.assertEqual(ports[2][0], 'insert')
                self.assertEqual(ports[2][1].addr, port3.addr)

            def _user_consume(self):
                pass

        self._create_comp(MySink)

    def test_gen_src_output_ports_getitem(self):
        port1 = None
        port2 = None
        port3 = None

        class MySource(
            bt2._UserSourceComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                nonlocal port1, port2, port3
                port1 = comp_self._add_output_port('clear')
                port2 = comp_self._add_output_port('print')
                port3 = comp_self._add_output_port('insert')

        comp = self._create_comp(MySource)
        self.assertEqual(port3.addr, comp.output_ports['insert'].addr)
        self.assertEqual(port2.addr, comp.output_ports['print'].addr)
        self.assertEqual(port1.addr, comp.output_ports['clear'].addr)
        del port1
        del port2
        del port3

    def test_gen_flt_output_ports_getitem(self):
        port1 = None
        port2 = None
        port3 = None

        class MyFilter(
            bt2._UserFilterComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                nonlocal port1, port2, port3
                port1 = comp_self._add_output_port('clear')
                port2 = comp_self._add_output_port('print')
                port3 = comp_self._add_output_port('insert')

        comp = self._create_comp(MyFilter)
        self.assertEqual(port3.addr, comp.output_ports['insert'].addr)
        self.assertEqual(port2.addr, comp.output_ports['print'].addr)
        self.assertEqual(port1.addr, comp.output_ports['clear'].addr)
        del port1
        del port2
        del port3

    def test_gen_flt_input_ports_getitem(self):
        port1 = None
        port2 = None
        port3 = None

        class MyFilter(
            bt2._UserFilterComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                nonlocal port1, port2, port3
                port1 = comp_self._add_input_port('clear')
                port2 = comp_self._add_input_port('print')
                port3 = comp_self._add_input_port('insert')

        comp = self._create_comp(MyFilter)
        self.assertEqual(port3.addr, comp.input_ports['insert'].addr)
        self.assertEqual(port2.addr, comp.input_ports['print'].addr)
        self.assertEqual(port1.addr, comp.input_ports['clear'].addr)
        del port1
        del port2
        del port3

    def test_gen_sink_input_ports_getitem(self):
        port1 = None
        port2 = None
        port3 = None

        class MySink(bt2._UserSinkComponent):
            def __init__(comp_self, config, params, obj):
                nonlocal port1, port2, port3
                port1 = comp_self._add_input_port('clear')
                port2 = comp_self._add_input_port('print')
                port3 = comp_self._add_input_port('insert')

            def _user_consume(self):
                pass

        comp = self._create_comp(MySink)
        self.assertEqual(port3.addr, comp.input_ports['insert'].addr)
        self.assertEqual(port2.addr, comp.input_ports['print'].addr)
        self.assertEqual(port1.addr, comp.input_ports['clear'].addr)
        del port1
        del port2
        del port3

    def test_gen_src_output_ports_getitem_invalid_key(self):
        class MySource(
            bt2._UserSourceComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                comp_self._add_output_port('clear')
                comp_self._add_output_port('print')
                comp_self._add_output_port('insert')

        comp = self._create_comp(MySource)

        with self.assertRaises(KeyError):
            comp.output_ports['hello']

    def test_gen_flt_output_ports_getitem_invalid_key(self):
        class MyFilter(
            bt2._UserFilterComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                comp_self._add_output_port('clear')
                comp_self._add_output_port('print')
                comp_self._add_output_port('insert')

        comp = self._create_comp(MyFilter)

        with self.assertRaises(KeyError):
            comp.output_ports['hello']

    def test_gen_flt_input_ports_getitem_invalid_key(self):
        class MyFilter(
            bt2._UserFilterComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                comp_self._add_input_port('clear')
                comp_self._add_input_port('print')
                comp_self._add_input_port('insert')

        comp = self._create_comp(MyFilter)

        with self.assertRaises(KeyError):
            comp.input_ports['hello']

    def test_gen_sink_input_ports_getitem_invalid_key(self):
        class MySink(bt2._UserSinkComponent):
            def __init__(comp_self, config, params, obj):
                comp_self._add_input_port('clear')
                comp_self._add_input_port('print')
                comp_self._add_input_port('insert')

                with self.assertRaises(KeyError):
                    comp_self._input_ports['hello']

            def _user_consume(self):
                pass

        comp = self._create_comp(MySink)

        with self.assertRaises(KeyError):
            comp.input_ports['hello']

    def test_gen_src_output_ports_len(self):
        class MySource(
            bt2._UserSourceComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                comp_self._add_output_port('clear')
                comp_self._add_output_port('print')
                comp_self._add_output_port('insert')

        comp = self._create_comp(MySource)
        self.assertEqual(len(comp.output_ports), 3)

    def test_gen_flt_output_ports_len(self):
        class MyFilter(
            bt2._UserFilterComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                comp_self._add_output_port('clear')
                comp_self._add_output_port('print')
                comp_self._add_output_port('insert')

        comp = self._create_comp(MyFilter)
        self.assertEqual(len(comp.output_ports), 3)

    def test_gen_flt_input_ports_len(self):
        class MyFilter(
            bt2._UserFilterComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                comp_self._add_input_port('clear')
                comp_self._add_input_port('print')
                comp_self._add_input_port('insert')

        comp = self._create_comp(MyFilter)
        self.assertEqual(len(comp.input_ports), 3)

    def test_gen_sink_input_ports_len(self):
        class MySink(bt2._UserSinkComponent):
            def __init__(comp_self, config, params, obj):
                comp_self._add_input_port('clear')
                comp_self._add_input_port('print')
                comp_self._add_input_port('insert')

            def _user_consume(self):
                pass

        comp = self._create_comp(MySink)
        self.assertEqual(len(comp.input_ports), 3)

    def test_gen_src_output_ports_iter(self):
        port1 = None
        port2 = None
        port3 = None

        class MySource(
            bt2._UserSourceComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                nonlocal port1, port2, port3
                port1 = comp_self._add_output_port('clear')
                port2 = comp_self._add_output_port('print')
                port3 = comp_self._add_output_port('insert')

        comp = self._create_comp(MySource)
        ports = []

        for port_name, port in comp.output_ports.items():
            ports.append((port_name, port))

        self.assertEqual(ports[0][0], 'clear')
        self.assertEqual(ports[0][1].addr, port1.addr)
        self.assertEqual(ports[1][0], 'print')
        self.assertEqual(ports[1][1].addr, port2.addr)
        self.assertEqual(ports[2][0], 'insert')
        self.assertEqual(ports[2][1].addr, port3.addr)
        del port1
        del port2
        del port3

    def test_gen_flt_output_ports_iter(self):
        port1 = None
        port2 = None
        port3 = None

        class MyFilter(
            bt2._UserFilterComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                nonlocal port1, port2, port3
                port1 = comp_self._add_output_port('clear')
                port2 = comp_self._add_output_port('print')
                port3 = comp_self._add_output_port('insert')

        comp = self._create_comp(MyFilter)
        ports = []

        for port_name, port in comp.output_ports.items():
            ports.append((port_name, port))

        self.assertEqual(ports[0][0], 'clear')
        self.assertEqual(ports[0][1].addr, port1.addr)
        self.assertEqual(ports[1][0], 'print')
        self.assertEqual(ports[1][1].addr, port2.addr)
        self.assertEqual(ports[2][0], 'insert')
        self.assertEqual(ports[2][1].addr, port3.addr)
        del port1
        del port2
        del port3

    def test_gen_flt_input_ports_iter(self):
        port1 = None
        port2 = None
        port3 = None

        class MyFilter(
            bt2._UserFilterComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                nonlocal port1, port2, port3
                port1 = comp_self._add_input_port('clear')
                port2 = comp_self._add_input_port('print')
                port3 = comp_self._add_input_port('insert')

        comp = self._create_comp(MyFilter)
        ports = []

        for port_name, port in comp.input_ports.items():
            ports.append((port_name, port))

        self.assertEqual(ports[0][0], 'clear')
        self.assertEqual(ports[0][1].addr, port1.addr)
        self.assertEqual(ports[1][0], 'print')
        self.assertEqual(ports[1][1].addr, port2.addr)
        self.assertEqual(ports[2][0], 'insert')
        self.assertEqual(ports[2][1].addr, port3.addr)
        del port1
        del port2
        del port3

    def test_gen_sink_input_ports_iter(self):
        port1 = None
        port2 = None
        port3 = None

        class MySink(bt2._UserSinkComponent):
            def __init__(comp_self, config, params, obj):
                nonlocal port1, port2, port3
                port1 = comp_self._add_input_port('clear')
                port2 = comp_self._add_input_port('print')
                port3 = comp_self._add_input_port('insert')

            def _user_consume(self):
                pass

        comp = self._create_comp(MySink)
        ports = []

        for port_name, port in comp.input_ports.items():
            ports.append((port_name, port))

        self.assertEqual(ports[0][0], 'clear')
        self.assertEqual(ports[0][1].addr, port1.addr)
        self.assertEqual(ports[1][0], 'print')
        self.assertEqual(ports[1][1].addr, port2.addr)
        self.assertEqual(ports[2][0], 'insert')
        self.assertEqual(ports[2][1].addr, port3.addr)
        del port1
        del port2
        del port3

    def test_name(self):
        class MySink(bt2._UserSinkComponent):
            def __init__(comp_self, config, params, obj):
                comp_self._add_input_port('clear')

            def _user_consume(self):
                pass

        comp = self._create_comp(MySink)
        self.assertEqual(comp.input_ports['clear'].name, 'clear')

    def test_connection_none(self):
        class MySink(bt2._UserSinkComponent):
            def __init__(comp_self, config, params, obj):
                comp_self._add_input_port('clear')

            def _user_consume(self):
                pass

        comp = self._create_comp(MySink)
        self.assertIsNone(comp.input_ports['clear'].connection)

    def test_is_connected_false(self):
        class MySink(bt2._UserSinkComponent):
            def __init__(comp_self, config, params, obj):
                comp_self._add_input_port('clear')

            def _user_consume(self):
                pass

        comp = self._create_comp(MySink)
        self.assertFalse(comp.input_ports['clear'].is_connected)

    def test_self_name(self):
        class MySink(bt2._UserSinkComponent):
            def __init__(comp_self, config, params, obj):
                port = comp_self._add_input_port('clear')
                self.assertEqual(port.name, 'clear')

            def _user_consume(self):
                pass

        self._create_comp(MySink)

    def test_self_connection_none(self):
        class MySink(bt2._UserSinkComponent):
            def __init__(comp_self, config, params, obj):
                port = comp_self._add_input_port('clear')
                self.assertIsNone(port.connection)

            def _user_consume(self):
                pass

        self._create_comp(MySink)

    def test_self_is_connected_false(self):
        class MySink(bt2._UserSinkComponent):
            def __init__(comp_self, config, params, obj):
                port = comp_self._add_input_port('clear')
                self.assertFalse(port.is_connected)

            def _user_consume(self):
                pass

        self._create_comp(MySink)

    def test_source_self_port_user_data(self):
        class MyUserData:
            def __del__(self):
                nonlocal objects_deleted
                objects_deleted += 1

        class MySource(
            bt2._UserFilterComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                nonlocal user_datas

                p = comp_self._add_output_port('port1')
                user_datas.append(p.user_data)
                p = comp_self._add_output_port('port2', 2)
                user_datas.append(p.user_data)
                p = comp_self._add_output_port('port3', MyUserData())
                user_datas.append(p.user_data)

        user_datas = []
        objects_deleted = 0

        comp = self._create_comp(MySource)
        self.assertEqual(len(user_datas), 3)
        self.assertIs(user_datas[0], None)
        self.assertEqual(user_datas[1], 2)
        self.assertIs(type(user_datas[2]), MyUserData)

        # Verify that the user data gets freed.
        self.assertEqual(objects_deleted, 0)
        del user_datas
        del comp
        self.assertEqual(objects_deleted, 1)

    def test_filter_self_port_user_data(self):
        class MyUserData:
            def __del__(self):
                nonlocal objects_deleted
                objects_deleted += 1

        class MyFilter(
            bt2._UserFilterComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            def __init__(comp_self, config, params, obj):
                nonlocal user_datas

                p = comp_self._add_output_port('port1')
                user_datas.append(p.user_data)
                p = comp_self._add_output_port('port2', 'user data string')
                user_datas.append(p.user_data)
                p = comp_self._add_output_port('port3', MyUserData())
                user_datas.append(p.user_data)

                p = comp_self._add_input_port('port4')
                user_datas.append(p.user_data)
                p = comp_self._add_input_port('port5', user_data={'user data': 'dict'})
                user_datas.append(p.user_data)
                p = comp_self._add_input_port('port6', MyUserData())
                user_datas.append(p.user_data)

        user_datas = []
        objects_deleted = 0

        comp = self._create_comp(MyFilter)
        self.assertEqual(len(user_datas), 6)
        self.assertIs(user_datas[0], None)
        self.assertEqual(user_datas[1], 'user data string')
        self.assertIs(type(user_datas[2]), MyUserData)
        self.assertIs(user_datas[3], None)
        self.assertEqual(user_datas[4], {'user data': 'dict'})
        self.assertIs(type(user_datas[5]), MyUserData)

        # Verify that the user data gets freed.
        self.assertEqual(objects_deleted, 0)
        del user_datas
        del comp
        self.assertEqual(objects_deleted, 2)

    def test_sink_self_port_user_data(self):
        class MyUserData:
            def __del__(self):
                nonlocal objects_deleted
                objects_deleted += 1

        class MySink(bt2._UserSinkComponent):
            def __init__(comp_self, config, params, obj):
                nonlocal user_datas

                p = comp_self._add_input_port('port1')
                user_datas.append(p.user_data)
                p = comp_self._add_input_port('port2', MyUserData())
                user_datas.append(p.user_data)

            def _user_consume(self):
                pass

        user_datas = []
        objects_deleted = 0

        comp = self._create_comp(MySink)
        self.assertEqual(len(user_datas), 2)
        self.assertIs(user_datas[0], None)
        self.assertIs(type(user_datas[1]), MyUserData)

        # Verify that the user data gets freed.
        self.assertEqual(objects_deleted, 0)
        del user_datas
        del comp
        self.assertEqual(objects_deleted, 1)


if __name__ == '__main__':
    unittest.main()



# SPDX-License-Identifier: GPL-2.0-only
#
# Copyright (C) 2020 EfficiOS Inc.
#

import bt2


class TheSourceIterator(bt2._UserMessageIterator):
    def __init__(self, config, port):
        tc, sc, ec = port.user_data

        trace = tc()
        stream = trace.create_stream(sc, name='the-stream')

        self._msgs = [
            self._create_stream_beginning_message(stream),
            self._create_event_message(ec, stream),
            self._create_stream_end_message(stream),
        ]

    def __next__(self):
        if len(self._msgs) == 0:
            raise StopIteration

        return self._msgs.pop(0)


@bt2.plugin_component_class
class TheSource(bt2._UserSourceComponent, message_iterator_class=TheSourceIterator):
    def __init__(self, config, params, obj):
        tc = self._create_trace_class()
        sc = tc.create_stream_class()
        ec = sc.create_event_class(name='the-event')
        self._add_output_port('out', user_data=(tc, sc, ec))


bt2.register_plugin(__name__, "foo")



# Copyright (c) 2016, Matt Layman

import argparse
import sys
import unittest

from tap.i18n import _
from tap.loader import Loader


def main(argv=sys.argv, stream=sys.stderr):
    """Entry point for ``tappy`` command."""
    args = parse_args(argv)
    suite = build_suite(args)
    runner = unittest.TextTestRunner(verbosity=args.verbose, stream=stream)
    result = runner.run(suite)

    return get_status(result)


def build_suite(args):
    """Build a test suite by loading TAP files or a TAP stream."""
    loader = Loader()
    if len(args.files) == 0 or args.files[0] == '-':
        suite = loader.load_suite_from_stdin()
    else:
        suite = loader.load(args.files)
    return suite


def parse_args(argv):
    description = _('A TAP consumer for Python')
    epilog = _(
        'When no files are given or a dash (-) is used for the file name, '
        'tappy will read a TAP stream from STDIN.')
    parser = argparse.ArgumentParser(description=description, epilog=epilog)
    parser.add_argument(
        'files', metavar='FILE', nargs='*', help=_(
            'A file containing TAP output. Any directories listed will be '
            'scanned for files to include as TAP files.'))
    parser.add_argument(
        '-v', '--verbose', action='store_const', default=1, const=2,
        help=_('use verbose messages'))

    # argparse expects the executable to be removed from argv.
    args = parser.parse_args(argv[1:])

    # When no files are provided, the user wants to use a TAP stream on STDIN.
    # But they probably didn't mean it if there is no pipe connected.
    # In that case, print the help and exit.
    if not args.files and sys.stdin.isatty():
        sys.exit(parser.print_help())

    return args


def get_status(result):
    """Get a return status from the result."""
    if result.wasSuccessful():
        return 0
    else:
        return 1



#
# Copyright (C) 2019 EfficiOS Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; only version 2
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

import unittest
import bt2


class MipTestCase(unittest.TestCase):
    def test_get_greatest_operative_mip_version(self):
        class Source1(
            bt2._UserSourceComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            @classmethod
            def _user_get_supported_mip_versions(cls, params, obj, log_level):
                return [0, 1]

        class Source2(
            bt2._UserSourceComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            @classmethod
            def _user_get_supported_mip_versions(cls, params, obj, log_level):
                return [0, 2]

        class Source3(
            bt2._UserSourceComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            pass

        descriptors = [
            bt2.ComponentDescriptor(Source1),
            bt2.ComponentDescriptor(Source2),
            bt2.ComponentDescriptor(Source3),
        ]
        version = bt2.get_greatest_operative_mip_version(descriptors)
        self.assertEqual(version, 0)

    def test_get_greatest_operative_mip_version_no_match(self):
        class Source1(
            bt2._UserSourceComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            @classmethod
            def _user_get_supported_mip_versions(cls, params, obj, log_level):
                return [0]

        class Source2(
            bt2._UserSourceComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            @classmethod
            def _user_get_supported_mip_versions(cls, params, obj, log_level):
                return [1]

        descriptors = [
            bt2.ComponentDescriptor(Source1),
            bt2.ComponentDescriptor(Source2),
        ]

        version = bt2.get_greatest_operative_mip_version(descriptors)
        self.assertIsNone(version)

    def test_get_greatest_operative_mip_version_empty_descriptors(self):
        with self.assertRaises(ValueError):
            bt2.get_greatest_operative_mip_version([])

    def test_get_greatest_operative_mip_version_wrong_descriptor_type(self):
        class Source1(
            bt2._UserSourceComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            @classmethod
            def _user_get_supported_mip_versions(cls, params, obj, log_level):
                return [0, 1]

        descriptors = [bt2.ComponentDescriptor(Source1), object()]

        with self.assertRaises(TypeError):
            bt2.get_greatest_operative_mip_version(descriptors)

    def test_get_greatest_operative_mip_version_wrong_log_level_type(self):
        class Source1(
            bt2._UserSourceComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            pass

        descriptors = [bt2.ComponentDescriptor(Source1)]

        with self.assertRaises(TypeError):
            bt2.get_greatest_operative_mip_version(descriptors, 'lel')

    def test_get_greatest_operative_mip_version_wrong_log_level_value(self):
        class Source1(
            bt2._UserSourceComponent, message_iterator_class=bt2._UserMessageIterator
        ):
            pass

        descriptors = [bt2.ComponentDescriptor(Source1)]

        with self.assertRaises(ValueError):
            bt2.get_greatest_operative_mip_version(descriptors, 12345)

    def test_get_maximal_mip_version(self):
        self.assertEqual(bt2.get_maximal_mip_version(), 0)


if __name__ == '__main__':
    unittest.main()



import bt2
import os

# Source component classes in this file recognize and group inputs in
# various ways.  One stream is created by each component, with a name
# derived from the component class and the inputs.  This is then checked
# using the `sink.text.details` sink.


class TestIter(bt2._UserMessageIterator):
    def __init__(self, config, output_port):
        inputs = output_port.user_data['inputs']
        sc = output_port.user_data['sc']
        tc = sc.trace_class
        t = tc()
        s = t.create_stream(sc, name=self._make_stream_name(inputs))

        self._msgs = [
            self._create_stream_beginning_message(s),
            self._create_stream_end_message(s),
        ]

    def _make_stream_name(self, inputs):
        comp_cls_name = self._component.__class__.__name__
        return (
            comp_cls_name
            + ': '
            + ', '.join(sorted([os.path.basename(str(x)) for x in inputs]))
        )

    def __next__(self):
        if len(self._msgs) == 0:
            raise StopIteration

        return self._msgs.pop(0)


class Base:
    def __init__(self, params):
        tc = self._create_trace_class()
        sc = tc.create_stream_class()

        self._add_output_port('out', {'inputs': params['inputs'], 'sc': sc})


@bt2.plugin_component_class
class TestSourceExt(Base, bt2._UserSourceComponent, message_iterator_class=TestIter):
    """
    Recognize files whose name start with 'aaa', 'bbb' or 'ccc'.

    'aaa' files are grouped together, 'bbb' files are grouped together, 'ccc'
    files are not grouped.
    """

    def __init__(self, config, params, obj):
        super().__init__(params)

    @staticmethod
    def _user_query(priv_query_exec, obj, params, method_obj):
        if obj == 'babeltrace.support-info':
            if params['type'] == 'file':
                name = os.path.basename(str(params['input']))

                if name.startswith('aaa'):
                    return {'weight': 1, 'group': 'aaa'}
                elif name.startswith('bbb'):
                    return {'weight': 0.5, 'group': 'bbb'}
                elif name.startswith('ccc'):
                    # Try two different ways of returning 1 (an int and a float).
                    if name[3] == '1':
                        return 1
                    else:
                        return 1.0

            return 0
        else:
            raise bt2.UnknownObject


@bt2.plugin_component_class
class TestSourceSomeDir(
    Base, bt2._UserSourceComponent, message_iterator_class=TestIter
):
    """Recognizes directories named "some-dir".  The file "aaa10" in the
    directory "some-dir" won't be found by TestSourceExt, because we won't
    recurse in "some-dir"."""

    def __init__(self, config, params, obj):
        super().__init__(params)

    @staticmethod
    def _user_query(priv_query_exec, obj, params, method_obj):
        if obj == 'babeltrace.support-info':
            if params['type'] == 'directory':
                name = os.path.basename(str(params['input']))
                return 1 if name == 'some-dir' else 0
            else:
                return 0
        else:
            raise bt2.UnknownObject


@bt2.plugin_component_class
class TestSourceABCDE(Base, bt2._UserSourceComponent, message_iterator_class=TestIter):
    """A source that recognizes the arbitrary string input "ABCDE"."""

    def __init__(self, config, params, obj):
        super().__init__(params)

    @staticmethod
    def _user_query(priv_query_exec, obj, params, method_obj):
        if obj == 'babeltrace.support-info':
            return (
                1.0
                if params['type'] == 'string' and params['input'] == 'ABCDE'
                else 0.0
            )
        else:
            raise bt2.UnknownObject


class TestSourceNoQuery(bt2._UserSourceComponent, message_iterator_class=TestIter):
    """A source that does not implement _query at all."""


bt2.register_plugin(module_name=__name__, name="test")



#
# Copyright (C) 2019 EfficiOS Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; only version 2
# of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

import unittest
import bt2
import utils
from utils import TestOutputPortMessageIterator

from bt2 import field as bt2_field
from bt2 import stream as bt2_stream
from bt2 import event_class as bt2_event_class
from bt2 import clock_snapshot as bt2_clock_snapshot


class EventTestCase(unittest.TestCase):
    def _create_test_const_event_message(
        self,
        packet_fields_config=None,
        event_fields_config=None,
        with_clockclass=False,
        with_cc=False,
        with_sc=False,
        with_ep=False,
        with_packet=False,
    ):
        class MyIter(bt2._UserMessageIterator):
            def __init__(self, config, self_output_port):
                self._at = 0
                self._msgs = [self._create_stream_beginning_message(test_obj.stream)]

                if with_packet:
                    assert test_obj.packet
                    self._msgs.append(
                        self._create_packet_beginning_message(test_obj.packet)
                    )

                default_clock_snapshot = 789 if with_clockclass else None

                if with_packet:
                    assert test_obj.packet
                    ev_parent = test_obj.packet
                else:
                    assert test_obj.stream
                    ev_parent = test_obj.stream

                msg = self._create_event_message(
                    test_obj.event_class, ev_parent, default_clock_snapshot
                )

                if event_fields_config is not None:
                    event_fields_config(msg.event)

                self._msgs.append(msg)

                if with_packet:
                    self._msgs.append(self._create_packet_end_message(test_obj.packet))

                self._msgs.append(self._create_stream_end_message(test_obj.stream))

            def __next__(self):
                if self._at == len(self._msgs):
                    raise bt2.Stop

                msg = self._msgs[self._at]
                self._at += 1
                return msg

        class MySrc(bt2._UserSourceComponent, message_iterator_class=MyIter):
            def __init__(self, config, params, obj):
                self._add_output_port('out')
                tc = self._create_trace_class()

                clock_class = None
                if with_clockclass:
                    clock_class = self._create_clock_class(frequency=1000)

                # event common context (stream-class-defined)
                cc = None
                if with_cc:
                    cc = tc.create_structure_field_class()
                    cc += [
                        ('cpu_id', tc.create_signed_integer_field_class(8)),
                        ('stuff', tc.create_double_precision_real_field_class()),
                    ]

                # packet context (stream-class-defined)
                pc = None

                if with_packet:
                    pc = tc.create_structure_field_class()
                    pc += [
                        ('something', tc.create_unsigned_integer_field_class(8)),
                        (
                            'something_else',
                            tc.create_double_precision_real_field_class(),
                        ),
                    ]

                stream_class = tc.create_stream_class(
                    default_clock_class=clock_class,
                    event_common_context_field_class=cc,
                    packet_context_field_class=pc,
                    supports_packets=with_packet,
                )

                # specific context (event-class-defined)
                sc = None
                if with_sc:
                    sc = tc.create_structure_field_class()
                    sc += [
                        ('ant', tc.create_signed_integer_field_class(16)),
                        ('msg', tc.create_string_field_class()),
                    ]

                # event payload
                ep = None
                if with_ep:
                    ep = tc.create_structure_field_class()
                    ep += [
                        ('giraffe', tc.create_signed_integer_field_class(32)),
                        ('gnu', tc.create_signed_integer_field_class(8)),
                        ('mosquito', tc.create_signed_integer_field_class(8)),
                    ]

                event_class = stream_class.create_event_class(
                    name='garou',
                    specific_context_field_class=sc,
                    payload_field_class=ep,
                )

                trace = tc()
                stream = trace.create_stream(stream_class)

                if with_packet:
                    packet = stream.create_packet()

                if packet_fields_config is not None:
                    assert packet
                    packet_fields_config(packet)

                if with_packet:
                    test_obj.packet = packet

                test_obj.stream = stream
                test_obj.event_class = event_class

        test_obj = self
        self._graph = bt2.Graph()
        self._src_comp = self._graph.add_component(MySrc, 'my_source')
        self._msg_iter = TestOutputPortMessageIterator(
            self._graph, self._src_comp.output_ports['out']
        )

        for msg in self._msg_iter:
            if type(msg) is bt2._EventMessageConst:
                self._event_msg = msg
                return msg

    def test_const_attr_event_class(self):
        msg = self._create_test_const_event_message()
        self.assertEqual(msg.event.cls.addr, self.event_class.addr)
        self.assertIs(type(msg.event.cls), bt2_event_class._EventClassConst)

    def test_attr_event_class(self):
        msg = utils.get_event_message()
        self.assertIs(type(msg.event.cls), bt2_event_class._EventClass)

    def test_const_attr_name(self):
        msg = self._create_test_const_event_message()
        self.assertEqual(msg.event.name, self.event_class.name)

    def test_const_attr_id(self):
        msg = self._create_test_const_event_message()
        self.assertEqual(msg.event.id, self.event_class.id)

    def test_const_get_common_context_field(self):
        def event_fields_config(event):
            event.common_context_field['cpu_id'] = 1
            event.common_context_field['stuff'] = 13.194

        msg = self._create_test_const_event_message(
            event_fields_config=event_fields_config, with_cc=True
        )

        self.assertEqual(msg.event.common_context_field['cpu_id'], 1)
        self.assertEqual(msg.event.common_context_field['stuff'], 13.194)
        self.assertIs(
            type(msg.event.common_context_field), bt2_field._StructureFieldConst
        )

    def test_attr_common_context_field(self):
        msg = utils.get_event_message()
        self.assertIs(type(msg.event.common_context_field), bt2_field._StructureField)

    def test_const_no_common_context_field(self):
        msg = self._create_test_const_event_message(with_cc=False)
        self.assertIsNone(msg.event.common_context_field)

    def test_const_get_specific_context_field(self):
        def event_fields_config(event):
            event.specific_context_field['ant'] = -1
            event.specific_context_field['msg'] = 'hellooo'

        msg = self._create_test_const_event_message(
            event_fields_config=event_fields_config, with_sc=True
        )

        self.assertEqual(msg.event.specific_context_field['ant'], -1)
        self.assertEqual(msg.event.specific_context_field['msg'], 'hellooo')
        self.assertIs(
            type(msg.event.specific_context_field), bt2_field._StructureFieldConst
        )

    def test_attr_specific_context_field(self):
        msg = utils.get_event_message()
        self.assertIs(type(msg.event.specific_context_field), bt2_field._StructureField)

    def test_const_no_specific_context_field(self):
        msg = self._create_test_const_event_message(with_sc=False)
        self.assertIsNone(msg.event.specific_context_field)

    def test_const_get_event_payload_field(self):
        def event_fields_config(event):
            event.payload_field['giraffe'] = 1
            event.payload_field['gnu'] = 23
            event.payload_field['mosquito'] = 42

        msg = self._create_test_const_event_message(
            event_fields_config=event_fields_config, with_ep=True
        )

        self.assertEqual(msg.event.payload_field['giraffe'], 1)
        self.assertEqual(msg.event.payload_field['gnu'], 23)
        self.assertEqual(msg.event.payload_field['mosquito'], 42)
        self.assertIs(type(msg.event.payload_field), bt2_field._StructureFieldConst)

    def test_attr_payload_field(self):
        msg = utils.get_event_message()
        self.assertIs(type(msg.event.payload_field), bt2_field._StructureField)

    def test_const_no_payload_field(self):
        msg = self._create_test_const_event_message(with_ep=False)
        self.assertIsNone(msg.event.payload_field)

    def test_const_clock_value(self):
        msg = self._create_test_const_event_message(with_clockclass=True)
        self.assertEqual(msg.default_clock_snapshot.value, 789)
        self.assertIs(
            type(msg.default_clock_snapshot), bt2_clock_snapshot._ClockSnapshotConst
        )

    def test_clock_value(self):
        msg = utils.get_event_message()
        self.assertEqual(msg.default_clock_snapshot.value, 789)
        self.assertIs(
            type(msg.default_clock_snapshot), bt2_clock_snapshot._ClockSnapshotConst
        )

    def test_const_no_clock_value(self):
        msg = self._create_test_const_event_message(with_clockclass=False)
        with self.assertRaisesRegex(
            ValueError, 'stream class has no default clock class'
        ):
            msg.default_clock_snapshot

    def test_const_stream(self):
        msg = self._create_test_const_event_message()
        self.assertEqual(msg.event.stream.addr, self.stream.addr)
        self.assertIs(type(msg.event.stream), bt2_stream._StreamConst)

    def test_stream(self):
        msg = utils.get_event_message()
        self.assertIs(type(msg.event.stream), bt2_stream._Stream)

    def test_const_getitem(self):
        def event_fields_config(event):
            event.payload_field['giraffe'] = 1
            event.payload_field['gnu'] = 23
            event.payload_field['mosquito'] = 42
            event.specific_context_field['ant'] = -1
            event.specific_context_field['msg'] = 'hellooo'
            event.common_context_field['cpu_id'] = 1
            event.common_context_field['stuff'] = 13.194

        def packet_fields_config(packet):
            packet.context_field['something'] = 154
            packet.context_field['something_else'] = 17.2

        msg = self._create_test_const_event_message(
            packet_fields_config=packet_fields_config,
            event_fields_config=event_fields_config,
            with_cc=True,
            with_sc=True,
            with_ep=True,
            with_packet=True,
        )
        ev = msg.event

        # Test event fields
        self.assertEqual(ev['giraffe'], 1)
        self.assertIs(type(ev['giraffe']), bt2_field._SignedIntegerFieldConst)
        self.assertEqual(ev['gnu'], 23)
        self.assertEqual(ev['mosquito'], 42)
        self.assertEqual(ev['ant'], -1)
        self.assertIs(type(ev['ant']), bt2_field._SignedIntegerFieldConst)
        self.assertEqual(ev['msg'], 'hellooo')
        self.assertEqual(ev['cpu_id'], 1)
        self.assertIs(type(ev['cpu_id']), bt2_field._SignedIntegerFieldConst)
        self.assertEqual(ev['stuff'], 13.194)

        # Test packet fields
        self.assertEqual(ev['something'], 154)
        self.assertIs(type(ev['something']), bt2_field._UnsignedIntegerFieldConst)
        self.assertEqual(ev['something_else'], 17.2)

        with self.assertRaises(KeyError):
            ev['yes']

    def test_const_getitem_no_packet(self):
        def event_fields_config(event):
            event.payload_field['giraffe'] = 1
            event.payload_field['gnu'] = 23
            event.payload_field['mosquito'] = 42

        msg = self._create_test_const_event_message(
            event_fields_config=event_fields_config,
            with_ep=True,
        )
        ev = msg.event

        with self.assertRaises(KeyError):
            ev['yes']

    def test_getitem(self):
        msg = utils.get_event_message()
        ev = msg.event
        self.assertEqual(ev['giraffe'], 1)
        self.assertIs(type(ev['giraffe']), bt2_field._SignedIntegerField)
        self.assertEqual(ev['ant'], -1)
        self.assertIs(type(ev['ant']), bt2_field._SignedIntegerField)
        self.assertEqual(ev['cpu_id'], 1)
        self.assertIs(type(ev['cpu_id']), bt2_field._SignedIntegerField)
        self.assertEqual(ev['something'], 154)
        self.assertIs(type(ev['something']), bt2_field._UnsignedIntegerField)


if __name__ == "__main__":
    unittest.main()
